var http = require('http');
var url = require('url');
var _ = require('underscore');

var GCM = require('gcm').GCM;
var apiKey = 'YOUR_BROWSER_API_KEY';
var gcm = new GCM(apiKey);

var apns = require('apn');
var options = {
    cert: 'apushplugin_cert.pem',                 /* Certificate file path */
    certData: null,                   /* String or Buffer containing certificate data, if supplied uses this instead of cert file path */
    key:  'apushplugin_key.pem',                  /* Key file path */
    keyData: null,                    /* String or Buffer containing key data, as certData */
    passphrase: null,                 /* A passphrase for the Key file */
    ca: null,                         /* String or Buffer of CA data to use for the TLS connection */
    pfx: null,                        /* File path for private key, certificate and CA certs in PFX or PKCS12 format. If supplied will be used instead of certificate and key above */
    pfxData: null,                    /* PFX or PKCS12 format data containing the private key, certificate and CA certs. If supplied will be used instead of loading from disk. */
    gateway: 'gateway.sandbox.push.apple.com', // 'gateway.push.apple.com',/* gateway address */
    port: 2195,                       /* gateway port */
    rejectUnauthorized: true,         /* Value of rejectUnauthorized property to be passed through to tls.connect() */
    enhanced: true,                   /* enable enhanced format */
    errorCallback: undefined, /* Callback when error occurs function(err,notification) */
    cacheLength: 100,                  /* Number of notifications to cache for error purposes */
    autoAdjustCache: true,            /* Whether the cache should grow in response to messages being lost after errors. */
    connectionTimeout: 0              /* The duration the socket should stay alive with no activity in milliseconds. 0 = Disabled. */
};
var apnsConnection = new apns.Connection(options);

apnsConnection.on('error', function(error) {
	console.log('Error with APN ' + error);
});

apnsConnection.on('transmitted', function(notification) {
	console.log('Notification sent to APN ' + notification.toString());
});

apnsConnection.on('connected', function() {
	console.log('Connected to APN.');
});

apnsConnection.on('disconnected', function() {
	console.log('Disconnected from APN.');
});

apnsConnection.on('transmissionError', function(errorCode, notification) {
	console.log("Transmision error with APN. Error code " + errorCode + " notification: " + notification.toString());
});

//##################

var allDevices = [];

http.createServer(function (request, response) {
	var currentDate = new Date();
	console.log("#Received request. " + currentDate.getHours() + ":" + currentDate.getMinutes() + ":" + currentDate.getSeconds());
	request.setEncoding('utf8');
	request.on('data', function(data) {
		var parsedUrl = url.parse(request.url);
		if (parsedUrl.href == "/reg") {
			var parsedData = JSON.parse(data);
			console.log("#Adding device with id : " + parsedData.token + " platform: " + parsedData.platform);
			if (parsedData.platform == "iPod touch" || parsedData.platform == "iPad" || parsedData.platform == "iPhone") {
				parsedData.platform = "iOS";
			};
			allDevices.push(parsedData);

			if (allDevices.length == 1)
				startSendingMessages();
		}
		else if (parsedUrl.href == "/unreg") {
			//TODO: implement

			//console.log("#Removing device with id : " + data);
			//allDevices.splice(allDevices.indexOf(data), 1);
		}
	});

  	response.writeHead(200, {'Content-Type': 'text/plain'});
  	response.end();

}).listen(80);

console.log('Server running at http://127.0.0.1:80');





function startSendingMessages() {
console.log("Start sending message to devices.");
	setInterval( function() {
			var count = 0;
			var currentDate = new Date();
			var textMessage = "Message sent on : " + currentDate.getHours() + ":" + currentDate.getMinutes() + ":" + currentDate.getSeconds();

			if (allDevices.length == 0)
				return;

			function sendToAndroidDevices() {
				var allAndroidTokens = _.chain(allDevices)
									.where({platform : 'Android'})
									.map(function(value) { return value.token })
									.value();

				if (allAndroidTokens.length == 0)
					return;

				var message = {
				    'registration_id' : allAndroidTokens, 
				    'data.message' :  textMessage,
				    'data.msgcnt' : count,
				};

				gcm.send(message, function(err, messageId){
				    if (err) {
				        console.log("Something has gone wrong with GCM send!");
				    } else {
				        console.log("Sent with message ID: " + messageId + " on GCM.");
				    }
				});

				count = count + 1;
			}

			function sendToiOSDevices() {
				var alliOSDeviceTokens = _.chain(allDevices)
										  .where({platform : 'iOS'})
										  .map(function(value) { return value.token })
										  .value();

				//console.log("ios device tokens length " + alliOSDeviceTokens.length);

				if (alliOSDeviceTokens.length == 0)
					return;

				_.each(alliOSDeviceTokens, function(token) {
					var device = new apns.Device(token);

					var note = new apns.Notification();

					note.expiry = Math.floor(Date.now() / 1000) + 3600; // Expires 1 hour from now.
					note.badge = 3;
					note.sound = "ping.aiff";
					note.alert = textMessage;
					//note.payload = {'messageFrom': 'Caroline'};
					note.device = device;

					apnsConnection.sendNotification(note);

				});

			}


			sendToAndroidDevices();
			sendToiOSDevices();

		},
		5000
	);
}